@@ -1,10 +1,20 @@
 """ Riemann zeta and related function. """
 
-from sympy.core import Function, S, sympify, pi, I
-from sympy.core.function import ArgumentIndexError
-from sympy.functions.combinatorial.numbers import bernoulli, factorial, harmonic
-from sympy.functions.elementary.exponential import log, exp_polar
+from sympy.core.add import Add
+from sympy.core.cache import cacheit
+from sympy.core.function import ArgumentIndexError, expand_mul, Function
+from sympy.core.numbers import pi, I, Integer
+from sympy.core.relational import Eq
+from sympy.core.singleton import S
+from sympy.core.symbol import Dummy
+from sympy.core.sympify import sympify
+from sympy.functions.combinatorial.numbers import bernoulli, factorial, genocchi, harmonic
+from sympy.functions.elementary.complexes import re, unpolarify, Abs, polar_lift
+from sympy.functions.elementary.exponential import log, exp_polar, exp
+from sympy.functions.elementary.integers import ceiling, floor
 from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.piecewise import Piecewise
+from sympy.polys.polytools import Poly
 
 ###############################################################################
 ###################### LERCH TRANSCENDENT #####################################
@@ -76,7 +86,7 @@ class lerchphi(Function):
     reduces to a sum of Hurwitz zeta functions:
 
     >>> expand_func(lerchphi(-1, s, a))
-    2**(-s)*zeta(s, a/2) - 2**(-s)*zeta(s, a/2 + 1/2)
+    zeta(s, a/2)/2**s - zeta(s, a/2 + 1/2)/2**s
 
     If $a=1$, the Lerch transcendent reduces to the polylogarithm:
 
@@ -112,13 +122,12 @@ class lerchphi(Function):
 
     .. [1] Bateman, H.; Erdelyi, A. (1953), Higher Transcendental Functions,
            Vol. I, New York: McGraw-Hill. Section 1.11.
-    .. [2] http://dlmf.nist.gov/25.14
+    .. [2] https://dlmf.nist.gov/25.14
     .. [3] https://en.wikipedia.org/wiki/Lerch_transcendent
 
     """
 
     def _eval_expand_func(self, **hints):
-        from sympy import exp, I, floor, Add, Poly, Dummy, exp_polar, unpolarify
         z, s, a = self.args
         if z == 1:
             return zeta(s, a)
@@ -157,9 +166,14 @@ class lerchphi(Function):
             m, n = S([a.p, a.q])
             zet = exp_polar(2*pi*I/n)
             root = z**(1/n)
-            return add + mul*n**(s - 1)*Add(
-                *[polylog(s, zet**k*root)._eval_expand_func(**hints)
-                  / (unpolarify(zet)**k*root)**m for k in range(n)])
+            up_zet = unpolarify(zet)
+            addargs = []
+            for k in range(n):
+                p = polylog(s, zet**k*root)
+                if isinstance(p, polylog):
+                    p = p._eval_expand_func(**hints)
+                addargs.append(p/(up_zet**k*root)**m)
+            return add + mul*n**(s - 1)*Add(*addargs)
 
         # TODO use minpoly instead of ad-hoc methods when issue 5888 is fixed
         if isinstance(z, exp) and (z.args[0]/(pi*I)).is_Rational or z in [-1, I, -I]:
@@ -187,7 +201,7 @@ class lerchphi(Function):
         else:
             raise ArgumentIndexError
 
-    def _eval_rewrite_helper(self, z, s, a, target):
+    def _eval_rewrite_helper(self, target):
         res = self._eval_expand_func()
         if res.has(target):
             return res
@@ -195,10 +209,10 @@ class lerchphi(Function):
             return self
 
     def _eval_rewrite_as_zeta(self, z, s, a, **kwargs):
-        return self._eval_rewrite_helper(z, s, a, zeta)
+        return self._eval_rewrite_helper(zeta)
 
     def _eval_rewrite_as_polylog(self, z, s, a, **kwargs):
-        return self._eval_rewrite_helper(z, s, a, polylog)
+        return self._eval_rewrite_helper(polylog)
 
 ###############################################################################
 ###################### POLYLOGARITHM ##########################################
@@ -277,26 +291,17 @@ class polylog(Function):
 
     @classmethod
     def eval(cls, s, z):
-        s, z = sympify((s, z))
-        if z is S.One:
-            return zeta(s)
-        elif z is S.NegativeOne:
-            return -dirichlet_eta(s)
-        elif z is S.Zero:
-            return S.Zero
-        elif s == 2:
-            if z == S.Half:
-                return pi**2/12 - log(2)**2/2
-            elif z == 2:
-                return pi**2/4 - I*pi*log(2)
-            elif z == -(sqrt(5) - 1)/2:
-                return -pi**2/15 + log((sqrt(5)-1)/2)**2/2
-            elif z == -(sqrt(5) + 1)/2:
-                return -pi**2/10 - log((sqrt(5)+1)/2)**2
-            elif z == (3 - sqrt(5))/2:
-                return pi**2/15 - log((sqrt(5)-1)/2)**2
-            elif z == (sqrt(5) - 1)/2:
-                return pi**2/10 - log((sqrt(5)-1)/2)**2
+        if z.is_number:
+            if z is S.One:
+                return zeta(s)
+            elif z is S.NegativeOne:
+                return -dirichlet_eta(s)
+            elif z is S.Zero:
+                return S.Zero
+            elif s == 2:
+                dilogtable = _dilogtable()
+                if z in dilogtable:
+                    return dilogtable[z]
 
         if z.is_zero:
             return S.Zero
@@ -320,8 +325,6 @@ class polylog(Function):
                 return z/(1 - z)
 
         # polylog is branched, but not over the unit disk
-        from sympy.functions.elementary.complexes import (Abs, unpolarify,
-                                                          polar_lift)
         if z.has(exp_polar, polar_lift) and (zone or (Abs(z) <= S.One) == True):
             return cls(s, unpolarify(z))
 
@@ -335,7 +338,6 @@ class polylog(Function):
         return z*lerchphi(z, s, 1)
 
     def _eval_expand_func(self, **hints):
-        from sympy import log, expand_mul, Dummy
         s, z = self.args
         if s == 1:
             return -log(1 - z)
@@ -352,6 +354,38 @@ class polylog(Function):
         if z.is_zero:
             return True
 
+    def _eval_nseries(self, x, n, logx, cdir=0):
+        from sympy.series.order import Order
+        nu, z = self.args
+
+        z0 = z.subs(x, 0)
+        if z0 is S.NaN:
+            z0 = z.limit(x, 0, dir='-' if re(cdir).is_negative else '+')
+
+        if z0.is_zero:
+            # In case of powers less than 1, number of terms need to be computed
+            # separately to avoid repeated callings of _eval_nseries with wrong n
+            try:
+                _, exp = z.leadterm(x)
+            except (ValueError, NotImplementedError):
+                return self
+
+            if exp.is_positive:
+                newn = ceiling(n/exp)
+                o = Order(x**n, x)
+                r = z._eval_nseries(x, n, logx, cdir).removeO()
+                if r is S.Zero:
+                    return o
+
+                term = r
+                s = [term]
+                for k in range(2, newn):
+                    term *= r
+                    s.append(term/k**nu)
+                return Add(*s) + o
+
+        return super(polylog, self)._eval_nseries(x, n, logx, cdir)
+
 ###############################################################################
 ###################### HURWITZ GENERALIZED ZETA FUNCTION ######################
 ###############################################################################
@@ -370,13 +404,10 @@ class zeta(Function):
     .. math:: \zeta(s, a) = \sum_{n=0}^\infty \frac{1}{(n + a)^s},
 
     where the standard choice of argument for $n + a$ is used. For fixed
-    $a$ with $\operatorname{Re}(a) > 0$ the Hurwitz zeta function admits a
-    meromorphic continuation to all of $\mathbb{C}$, it is an unbranched
+    $a$ not a nonpositive integer the Hurwitz zeta function admits a
+    meromorphic continuation to all of $\mathbb{C}$; it is an unbranched
     function with a simple pole at $s = 1$.
 
-    Analytic continuation to other $a$ is possible under some circumstances,
-    but this is not typically done.
-
     The Hurwitz zeta function is a special case of the Lerch transcendent:
 
     .. math:: \zeta(s, a) = \Phi(1, s, a).
@@ -385,8 +416,8 @@ class zeta(Function):
     $s$ and $a$ (also $\operatorname{Re}(a) < 0$), see the documentation of
     :class:`lerchphi` for a description of the branching behavior.
 
-    If no value is passed for $a$, by this function assumes a default value
-    of $a = 1$, yielding the Riemann zeta function.
+    If no value is passed for $a$ a default value of $a = 1$ is assumed,
+    yielding the Riemann zeta function.
 
     Examples
     ========
@@ -410,25 +441,24 @@ class zeta(Function):
     >>> zeta(s).rewrite(dirichlet_eta)
     dirichlet_eta(s)/(1 - 2**(1 - s))
 
-    The Riemann zeta function at positive even integer and negative odd integer
-    values is related to the Bernoulli numbers:
+    The Riemann zeta function at nonnegative even and negative integer
+    values is related to the Bernoulli numbers and polynomials:
 
     >>> zeta(2)
     pi**2/6
     >>> zeta(4)
     pi**4/90
+    >>> zeta(0)
+    -1/2
     >>> zeta(-1)
     -1/12
+    >>> zeta(-4)
+    0
 
     The specific formulae are:
 
-    .. math:: \zeta(2n) = (-1)^{n+1} \frac{B_{2n} (2\pi)^{2n}}{2(2n)!}
-    .. math:: \zeta(-n) = -\frac{B_{n+1}}{n+1}
-
-    At negative even integers the Riemann zeta function is zero:
-
-    >>> zeta(-4)
-    0
+    .. math:: \zeta(2n) = -\frac{(2\pi i)^{2n} B_{2n}}{2(2n)!}
+    .. math:: \zeta(-n,a) = -\frac{B_{n+1}(a)}{n+1}
 
     No closed-form expressions are known at positive odd integers, but
     numerical evaluation is possible:
@@ -463,50 +493,42 @@ class zeta(Function):
     References
     ==========
 
-    .. [1] http://dlmf.nist.gov/25.11
+    .. [1] https://dlmf.nist.gov/25.11
     .. [2] https://en.wikipedia.org/wiki/Hurwitz_zeta_function
 
     """
 
     @classmethod
-    def eval(cls, z, a_=None):
-        if a_ is None:
-            z, a = list(map(sympify, (z, 1)))
-        else:
-            z, a = list(map(sympify, (z, a_)))
+    def eval(cls, s, a=None):
+        if a is S.One:
+            return cls(s)
+        elif s is S.NaN or a is S.NaN:
+            return S.NaN
+        elif s is S.One:
+            return S.ComplexInfinity
+        elif s is S.Infinity:
+            return S.One
+        elif a is S.Infinity:
+            return S.Zero
 
-        if a.is_Number:
-            if a is S.NaN:
-                return S.NaN
-            elif a is S.One and a_ is not None:
-                return cls(z)
-            # TODO Should a == 0 return S.NaN as well?
+        sint = s.is_Integer
+        if a is None:
+            a = S.One
+        if sint and s.is_nonpositive:
+            return bernoulli(1-s, a) / (s-1)
+        elif a is S.One:
+            if sint and s.is_even:
+                return -(2*pi*I)**s * bernoulli(s) / (2*factorial(s))
+        elif sint and a.is_Integer and a.is_positive:
+            return cls(s) - harmonic(a-1, s)
+        elif a.is_Integer and a.is_nonpositive and \
+                (s.is_integer is False or s.is_nonpositive is False):
+            return S.NaN
 
-        if z.is_Number:
-            if z is S.NaN:
-                return S.NaN
-            elif z is S.Infinity:
-                return S.One
-            elif z.is_zero:
-                return S.Half - a
-            elif z is S.One:
-                return S.ComplexInfinity
-        if z.is_integer:
-            if a.is_Integer:
-                if z.is_negative:
-                    zeta = (-1)**z * bernoulli(-z + 1)/(-z + 1)
-                elif z.is_even and z.is_positive:
-                    B, F = bernoulli(z), factorial(z)
-                    zeta = ((-1)**(z/2+1) * 2**(z - 1) * B * pi**z) / F
-                else:
-                    return
-
-                if a.is_negative:
-                    return zeta + harmonic(abs(a), z)
-                else:
-                    return zeta - harmonic(a - 1, z)
-        if z.is_zero:
-            return S.Half - a
+    def _eval_rewrite_as_bernoulli(self, s, a=1, **kwargs):
+        if a == 1 and s.is_integer and s.is_nonnegative and s.is_even:
+            return -(2*pi*I)**s * bernoulli(s) / (2*factorial(s))
+        return bernoulli(1-s, a) / (s-1)
 
     def _eval_rewrite_as_dirichlet_eta(self, s, a=1, **kwargs):
         if a != 1:
@@ -522,6 +544,17 @@ class zeta(Function):
         if arg_is_one is not None:
             return not arg_is_one
 
+    def _eval_expand_func(self, **hints):
+        s = self.args[0]
+        a = self.args[1] if len(self.args) > 1 else S.One
+        if a.is_integer:
+            if a.is_positive:
+                return zeta(s) - harmonic(a-1, s)
+            if a.is_nonpositive and (s.is_integer is False or
+                    s.is_nonpositive is False):
+                return S.NaN
+        return self
+
     def fdiff(self, argindex=1):
         if len(self.args) == 2:
             s, a = self.args
@@ -532,6 +565,22 @@ class zeta(Function):
         else:
             raise ArgumentIndexError
 
+    def _eval_as_leading_term(self, x, logx=None, cdir=0):
+        if len(self.args) == 2:
+            s, a = self.args
+        else:
+            s, a = self.args + (S.One,)
+
+        try:
+            c, e = a.leadterm(x)
+        except NotImplementedError:
+            return self
+
+        if e.is_negative and not s.is_positive:
+            raise NotImplementedError
+
+        return super(zeta, self)._eval_as_leading_term(x, logx, cdir)
+
 
 class dirichlet_eta(Function):
     r"""
@@ -540,22 +589,31 @@ class dirichlet_eta(Function):
     Explanation
     ===========
 
-    For $\operatorname{Re}(s) > 0$, this function is defined as
+    For $\operatorname{Re}(s) > 0$ and $0 < x \le 1$, this function is defined as
 
-    .. math:: \eta(s) = \sum_{n=1}^\infty \frac{(-1)^{n-1}}{n^s}.
+    .. math:: \eta(s, a) = \sum_{n=0}^\infty \frac{(-1)^n}{(n+a)^s}.
 
-    It admits a unique analytic continuation to all of $\mathbb{C}$.
-    It is an entire, unbranched function.
+    It admits a unique analytic continuation to all of $\mathbb{C}$ for any
+    fixed $a$ not a nonpositive integer. It is an entire, unbranched function.
+
+    It can be expressed using the Hurwitz zeta function as
+
+    .. math:: \eta(s, a) = \zeta(s,a) - 2^{1-s} \zeta\left(s, \frac{a+1}{2}\right)
+
+    and using the generalized Genocchi function as
+
+    .. math:: \eta(s, a) = \frac{G(1-s, a)}{2(s-1)}.
+
+    In both cases the limiting value of $\log2 - \psi(a) + \psi\left(\frac{a+1}{2}\right)$
+    is used when $s = 1$.
 
     Examples
     ========
 
-    The Dirichlet eta function is closely related to the Riemann zeta function:
-
     >>> from sympy import dirichlet_eta, zeta
     >>> from sympy.abc import s
     >>> dirichlet_eta(s).rewrite(zeta)
-    (1 - 2**(1 - s))*zeta(s)
+    Piecewise((log(2), Eq(s, 1)), ((1 - 2**(1 - s))*zeta(s), True))
 
     See Also
     ========
@@ -566,19 +624,84 @@ class dirichlet_eta(Function):
     ==========
 
     .. [1] https://en.wikipedia.org/wiki/Dirichlet_eta_function
+    .. [2] Peter Luschny, "An introduction to the Bernoulli function",
+           https://arxiv.org/abs/2009.06743
 
     """
 
+    @classmethod
+    def eval(cls, s, a=None):
+        if a is S.One:
+            return cls(s)
+        if a is None:
+            if s == 1:
+                return log(2)
+            z = zeta(s)
+            if not z.has(zeta):
+                return (1 - 2**(1-s)) * z
+            return
+        elif s == 1:
+            from sympy.functions.special.gamma_functions import digamma
+            return log(2) - digamma(a) + digamma((a+1)/2)
+        z1 = zeta(s, a)
+        z2 = zeta(s, (a+1)/2)
+        if not z1.has(zeta) and not z2.has(zeta):
+            return z1 - 2**(1-s) * z2
+
+    def _eval_rewrite_as_zeta(self, s, a=1, **kwargs):
+        from sympy.functions.special.gamma_functions import digamma
+        if a == 1:
+            return Piecewise((log(2), Eq(s, 1)), ((1 - 2**(1-s)) * zeta(s), True))
+        return Piecewise((log(2) - digamma(a) + digamma((a+1)/2), Eq(s, 1)),
+                (zeta(s, a) - 2**(1-s) * zeta(s, (a+1)/2), True))
+
+    def _eval_rewrite_as_genocchi(self, s, a=S.One, **kwargs):
+        from sympy.functions.special.gamma_functions import digamma
+        return Piecewise((log(2) - digamma(a) + digamma((a+1)/2), Eq(s, 1)),
+                (genocchi(1-s, a) / (2 * (s-1)), True))
+
+    def _eval_evalf(self, prec):
+        if all(i.is_number for i in self.args):
+            return self.rewrite(zeta)._eval_evalf(prec)
+
+
+class riemann_xi(Function):
+    r"""
+    Riemann Xi function.
+
+    Examples
+    ========
+
+    The Riemann Xi function is closely related to the Riemann zeta function.
+    The zeros of Riemann Xi function are precisely the non-trivial zeros
+    of the zeta function.
+
+    >>> from sympy import riemann_xi, zeta
+    >>> from sympy.abc import s
+    >>> riemann_xi(s).rewrite(zeta)
+    s*(s - 1)*gamma(s/2)*zeta(s)/(2*pi**(s/2))
+
+    References
+    ==========
+
+    .. [1] https://en.wikipedia.org/wiki/Riemann_Xi_function
+
+    """
+
+
     @classmethod
     def eval(cls, s):
-        if s == 1:
-            return log(2)
+        from sympy.functions.special.gamma_functions import gamma
         z = zeta(s)
-        if not z.has(zeta):
-            return (1 - 2**(1 - s))*z
+        if s in (S.Zero, S.One):
+            return S.Half
+
+        if not isinstance(z, zeta):
+            return s*(s - 1)*gamma(s/2)*z/(2*pi**(s/2))
 
     def _eval_rewrite_as_zeta(self, s, **kwargs):
-        return (1 - 2**(1 - s)) * zeta(s)
+        from sympy.functions.special.gamma_functions import gamma
+        return s*(s - 1)*gamma(s/2)*zeta(s)/(2*pi**(s/2))
 
 
 class stieltjes(Function):
@@ -620,8 +743,6 @@ class stieltjes(Function):
 
     @classmethod
     def eval(cls, n, a=None):
-        n = sympify(n)
-
         if a is not None:
             a = sympify(a)
             if a is S.NaN:
@@ -647,3 +768,20 @@ class stieltjes(Function):
 
         if n.is_integer == False:
             return S.ComplexInfinity
+
+
+@cacheit
+def _dilogtable():
+    return {
+        S.Half: pi**2/12 - log(2)**2/2,
+        Integer(2) : pi**2/4 - I*pi*log(2),
+        -(sqrt(5) - 1)/2 : -pi**2/15 + log((sqrt(5)-1)/2)**2/2,
+        -(sqrt(5) + 1)/2 : -pi**2/10 - log((sqrt(5)+1)/2)**2,
+        (3 - sqrt(5))/2 : pi**2/15 - log((sqrt(5)-1)/2)**2,
+        (sqrt(5) - 1)/2 : pi**2/10 - log((sqrt(5)-1)/2)**2,
+        I : I*S.Catalan - pi**2/48,
+        -I : -I*S.Catalan - pi**2/48,
+        1 - I : pi**2/16 - I*S.Catalan - pi*I/4*log(2),
+        1 + I : pi**2/16 + I*S.Catalan + pi*I/4*log(2),
+        (1 - I)/2 : -log(2)**2/8 + pi*I*log(2)/8 + 5*pi**2/96 - I*S.Catalan
+    }
